package com.nisum.reactive;

import static org.junit.Assert.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.BDDMockito;
import org.mockito.Mockito;
import org.springframework.boot.test.mock.mockito.MockBean;

import com.nisum.reactive.dto.ProductDto;
import com.nisum.reactive.entity.Product;
import com.nisum.reactive.repository.ProductRepository;
import com.nisum.reactive.service.ProductService;
import com.nisum.reactive.util.AppUtils;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

public class ProductServiceTest {

	private ProductRepository productRepository;

	@MockBean
	private ProductService productService;

	@BeforeEach
	public void setUp() {
		productRepository = Mockito.mock(ProductRepository.class);
		productService = Mockito.mock(ProductService.class);
	}

	@Test
	public void getProductsTest() {
		Flux<Product> productFlux = Flux.just(new Product("102", "mobile", 1, 10000),
				new Product("103", "TV", 1, 50000));
		Flux<ProductDto> productDtoFlux = Flux.just(new ProductDto("102", "mobile", 1, 10000),
				new ProductDto("103", "TV", 1, 50000));
		// BDDMockito.given(productRepository.findAll()).willReturn(productFlux);
		BDDMockito.given(productService.getProducts()).willReturn(productDtoFlux);
		Flux<ProductDto> result = productService.getProducts();
		System.out.println(result);

		StepVerifier.create(result)
					.expectSubscription()
					.expectNextMatches(p -> p.getName().equals("mobile"))
					.expectNextMatches(p -> p.getName().equals("TV"))
					.verifyComplete();

	}
	
	@Test
	public void getProductTest() {
		Mono<Product> productFlux = Mono.just(new Product("102", "mobile", 1, 10000));
		Mono<ProductDto> productDtoFlux = Mono.just(new ProductDto("102", "mobile", 1, 10000));
		// BDDMockito.given(productRepository.findAll()).willReturn(productFlux);
		BDDMockito.given(productService.getProduct(any())).willReturn(productDtoFlux);
		Mono<ProductDto> result = productService.getProduct("102");
		System.out.println(result);

		StepVerifier.create(result)
					.expectSubscription()
					.expectNextMatches(p -> p.getName().equals("mobile"))
					.verifyComplete();

	}

}
