package com.nisum.reactive;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyDouble;
import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.when;

import org.junit.jupiter.api.Test;
import org.mockito.BDDMockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.reactive.server.WebTestClient;

import com.nisum.reactive.controller.ProductController;
import com.nisum.reactive.dto.ProductDto;
import com.nisum.reactive.service.ProductService;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@WebFluxTest(controllers = ProductController.class)
public class SpringBootApplicationTest {

	@MockBean
	private ProductService productService;

	@Autowired
	private WebTestClient webTestClient;

	
	
	@Test
	public void addProductTest() {
		Mono<ProductDto> productDtoMono = Mono.just(new ProductDto("102", "mobile", 1, 10000));
		when(productService.saveProduct(productDtoMono)).thenReturn(productDtoMono);

		webTestClient.post().uri("/products").body(Mono.just(productDtoMono), ProductDto.class).exchange()
				.expectStatus().isOk();// 200

	}
	
	@Test
	public void getProductsTest() throws Exception {
		Flux<ProductDto> productDtoFlux = Flux.just(new ProductDto("102", "mobile", 1, 10000),
				new ProductDto("103", "TV", 1, 50000));
		BDDMockito.given(productService.getProducts()).willReturn(productDtoFlux);
		/*
		 * import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
		 * import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
		 * @Autowired  private MockMvc mockMvc;
		 * mockMvc.perform(get("/products")) .andExpect(status().isOk());
		 */

		webTestClient.get().uri("/products").exchange().expectStatus().isOk();
	}
	
	@Test
	public void getProductTest(){
		Mono<ProductDto> productDtoMono=Mono.just(new ProductDto("102","mobile",1,10000));
		when(productService.getProduct(any())).thenReturn(productDtoMono);

		webTestClient.get().uri("/products/102")
				.exchange()
				.expectStatus().isOk();

	}
	@Test
	public void getProductsBetweenRange() {
		Flux<ProductDto> productDtoFlux = Flux.just(new ProductDto("102", "mobile", 1, 10000),
				new ProductDto("103", "TV", 1, 50000));
		
		BDDMockito.given(productService.getProductInRange(anyDouble(), anyDouble())).willReturn(productDtoFlux);
		webTestClient.get().uri("/products/product-range?min=1000&max=10000")
		.exchange()
		.expectStatus().isOk();
	}
	
	@Test
	public void updateProductTest(){
		Mono<ProductDto> productDtoMono=Mono.just(new ProductDto("102","mobile",1,10000));
		when(productService.updateProduct(productDtoMono,"102")).thenReturn(productDtoMono);

		webTestClient.put().uri("/products/update/102")
				.body(Mono.just(productDtoMono),ProductDto.class)
				.exchange()
				.expectStatus().isOk();//200
	}

	@Test
	public void deleteProductTest(){
    	given(productService.deleteProduct(any())).willReturn(Mono.empty());
		webTestClient.delete().uri("/products/delete/102")
				.exchange()
				.expectStatus().isOk();//200
	}

}
